@@ -1,27 +1,34 @@
 #!/bin/sh
-# next lines for bash, ignored by tclsh, restarting in background\
-export PATH=/bin:/usr/bin; \
-if [ ! -e "/usr/bin/tclsh" ]; then \
-	logger -p syslog.error "usb_modeswitch: tcl shell not found, install tcl package!"; \
-fi; \
-(/usr/bin/tclsh "$0" "$@" >/dev/null 2>&1 &); \
-sleep 1; \
-exit
+# next lines for bash, ignored by tclsh, restarting in background \
+( \
+count=120; \
+while [ $count != 0 ]; do \
+	if [ ! -e "/usr/bin/tclsh" ]; then \
+		sleep 1; \
+		count=$(($count - 1)); \
+	else \
+		exec /usr/bin/tclsh "$0" "$@" 2>/dev/null & \
+		exit 0; \
+	fi; \
+done; \
+) & \
+exit 0
 
 
 # Wrapper (tcl) for usb_modeswitch, called from
 # /lib/udev/rules.d/40-usb_modeswitch.rules
-# (part of data pack "usb-modeswitch-data")
+# (part of data pack "usb-modeswitch-data") via
+# /lib/udev/usb_modeswitch
 #
 # Does ID check on hotplugged USB devices and calls the
 # mode switching program with the matching parameter file
 # from /etc/usb_modeswitch.d
 #
-# Part of usb-modeswitch-1.1.3 package
+# Part of usb-modeswitch-1.1.4 package
 # (C) Josua Dietze 2009, 2010
 
 
-# Setting of the following switches is done in an external config
+# Setting of the these switches is done in the global config
 # file (/etc/usb_modeswitch.conf)
 
 set logging 0
@@ -35,7 +42,7 @@ set env(PATH) "/bin:/usr/bin"
 
 proc {Main} {argc argv} {
 
-global scsi usb match wc device logging noswitching
+global scsi usb config match wc device logging noswitching settings
 
 # The facility to add a symbolic link pointing to the
 # ttyUSB port which provides interrupt transfer, i.e.
@@ -44,29 +51,29 @@ global scsi usb match wc device logging noswitching
 # This is run once for every device interface by an
 # udev rule
 
-if {[lindex $argv 0] == "symlink"} {
-#	puts "symlink: udev path is [lindex $argv 2]"
+if {[lindex $argv 0] == "--symlink-name"} {
 #	set device [clock clicks]
-#	set logging 1
-#	Log "symlink: udev path is [lindex $argv 2]"
-	if [llength [glob -nocomplain /tmp/gsmmodem_*]] {
-		set sl [SymLinkName [lindex $argv 2]]
-#		Log "symlink name is :$sl:"
-		puts $sl
-#		puts [SymLinkName [lindex $argv 2]]
-	} else {
-		puts ""
-	}
+	puts [SymLinkName [lindex $argv 1]]
 	SafeExit
 }
 
-set dbdir	/etc/usb_modeswitch.d
+# The facility to bind the driver on-the-fly after a warm
+# boot; the device is still in modem mode but if the
+# driver was bound by the switching script before (ID
+# not yet added to the driver), the device needs to be
+# bound again
+
+if {[lindex $argv 0] == "--driver-bind"} {
+	CheckDriverBind [lindex $argv 1] [lindex $argv 2] [lindex $argv 3] [lindex $argv 4]
+	SafeExit
+}
+
+set settings(dbdir)	/etc/usb_modeswitch.d
 set bindir	/usr/sbin
 
 set devList1 {}
 set devList2 {}
 
-
 # argv contains the values provided from the udev rule
 # separated by "/"
 
@@ -78,7 +85,7 @@ if [string length [lindex $argList 1]] {
 	set device "noname"
 }
 
-ParseConfigFile
+ParseGlobalConfig
 
 Log "raw args from udev: $argv"
 
@@ -151,34 +158,25 @@ Log "----------------"
 
 if $noswitching {
 	Log "\nSwitching globally disabled. Exiting\n"
-	catch {exec logger -p syslog.notice "usb_modeswitch: switching disabled, no action for $usb(idVendor):$usb(idProduct)"}
+	catch {exec logger -p syslog.notice "usb_modeswitch: switching disabled, no action for $usb(idVendor):$usb(idProduct)" 2>/dev/null}
 	SafeExit
 }
 
-# Special ZTE check
-if {"$usb(idVendor)$usb(idProduct)" == "19d22000"} {
-	foreach dir {/etc/udev/rules.d /lib/udev/rules.d} {
-		catch {eval exec grep {"19d2.*2000.*eject"} [glob -nocomplain $dir/*]} result
-		if [regexp {(.*?):.*19d2} $result d ruleFile] {
-			Log "\nWarning: existing ZTE rule found in $ruleFile. Might cause problems\n"
-		}
-	}
-}
-
 # Check if there is more than one config file for this USB ID,
 # which would point to a possible ambiguity. If so, check if
 # SCSI values are needed
 
-set configList [glob -nocomplain $dbdir/$usb(idVendor):$usb(idProduct)*]
+set configList [ConfigGet list $usb(idVendor):$usb(idProduct)]
+
 if {[llength $configList] == 0} {
 	Log "Aargh! Config file missing for $usb(idVendor):$usb(idProduct)! Exiting"
 	SafeExit
 }
 
-set scsiNeeded false
+set scsiNeeded 0
 if {[llength $configList] > 1} {
 	if [regexp {:s} $configList] {
-		set scsiNeeded true
+		set scsiNeeded 1
 	}
 }
 if {!$scsiNeeded} {
@@ -262,7 +260,7 @@ if $scsiNeeded {
 # storage driver, so it's just the last resort
 
 if {$scsiNeeded && $scsi(vendor)==""} {
-	set testSCSI [exec $bindir/usb_modeswitch -v 0x$usb(idVendor) -p 0x$usb(idProduct)]
+	set testSCSI [exec $bindir/usb_modeswitch -v 0x$usb(idVendor) -p 0x$usb(idProduct) 2>/dev/null]
 	regexp {  Vendor String: (.*?)\n} $testSCSI d scsi(vendor)
 	regexp {   Model String: (.*?)\n} $testSCSI d scsi(model)
 	regexp {Revision String: (.*?)\n} $testSCSI d scsi(rev)
@@ -284,7 +282,7 @@ if {$scsiNeeded && $scsi(vendor)==""} {
 # end and provide a fallback
 
 set report {}
-set configList [glob -nocomplain $dbdir/$usb(idVendor):$usb(idProduct)*]
+#set configList [glob -nocomplain $settings(dbdir)/$usb(idVendor):$usb(idProduct)*]
 foreach configuration [lsort -decreasing $configList] {
 
 	# skipping installer leftovers
@@ -292,24 +290,31 @@ foreach configuration [lsort -decreasing $configList] {
 
 	Log "checking config: $configuration"
 	if [MatchDevice $configuration] {
-		set switch_config $configuration
+		ParseDeviceConfig [ConfigGet config $configuration]
 		set devList1 [glob -nocomplain /dev/ttyUSB* /dev/ttyACM* /dev/ttyHS*]
-		Log "! matched, now switching"
-		set tc [open /tmp/gsmmodem_$dev_top w]
-		close $tc
+		if {$config(waitBefore) == ""} {
+			Log "! matched, now switching"
+		} else {
+			Log "! matched, waiting time set to $config(waitBefore) seconds"
+			after [expr $config(waitBefore) * 1000]
+			Log " waiting is over, switching starts now"
+		}
 
 		# Now we are actually switching
 		if $logging {
-			Log " (running command: $bindir/usb_modeswitch -I -W -c $configuration)"
-			set report [exec $bindir/usb_modeswitch -I -W -D -c $configuration 2>@ stdout]
+			Log " (running command: $bindir/usb_modeswitch -I -W -c $settings(tmpConfig))"
+			set report [exec $bindir/usb_modeswitch -I -W -D -c $settings(tmpConfig) 2>@ stdout]
 		} else {
-			set report [exec $bindir/usb_modeswitch -I -Q -D -c $configuration]
+			set report [exec $bindir/usb_modeswitch -I -Q -D -c $settings(tmpConfig) 2>/dev/null]
 		}
 		Log "\nverbose output of usb_modeswitch:"
 		Log "--------------------------------"
 		Log $report
 		Log "--------------------------------"
 		Log "(end of usb_modeswitch output)\n"
+		if [regexp {/tmp/} $settings(tmpConfig)] {
+			file delete  $settings(tmpConfig)
+		}
 		break
 	} else {
 		Log "* no match, not switching with this config"
@@ -327,24 +332,15 @@ if [regexp -nocase {ok:[0-9a-f]{4}:[0-9a-f]{4}} $report] {
 
 	# For general driver loading; TODO: add respective device names.
 	# Presently only useful for HSO devices (which are recounted now)
-	set driverModule ""
-	set driverIDPath ""
-	set rc [open $configuration r]
-	set lineList [split [read $rc] \n]
-	close $rc
-	foreach line $lineList {
-		regexp {DriverModule[[:blank:]]*=[[:blank:]]*"?(\w+)"?} $line d driverModule
-		regexp {DriverIDPath[[:blank:]]*=[[:blank:]]*?"?([/\-\w]+)"?} $line d driverIDPath
-	}
-	if {$driverModule == ""} {
-		set driverModule "option"
-		set driverIDPath "/sys/bus/usb-serial/drivers/option1"
+	if {$config(driverModule) == ""} {
+		set config(driverModule) "option"
+		set config(driverIDPath) "/sys/bus/usb-serial/drivers/option1"
 	} else {
-		if {$driverIDPath == ""} {
-			set driverIDPath "/sys/bus/usb/drivers/$driverModule"
+		if {$config(driverIDPath) == ""} {
+			set config(driverIDPath) "/sys/bus/usb/drivers/$config(driverModule)"
 		}
 	}
-	Log "Driver module is \"$driverModule\", ID path is $driverIDPath\n"
+	Log "Driver module is \"$config(driverModule)\", ID path is $config(driverIDPath)\n"
 
 	# some settling time in ms
 	after 500
@@ -371,28 +367,29 @@ if [regexp -nocase {ok:[0-9a-f]{4}:[0-9a-f]{4}} $report] {
 		}
 		set t "$usb(idVendor)$usb(idProduct)"
 		if {[string length $t] == 8 && [string trim $t 0] != ""} {
-			set idfile $driverIDPath/new_id
+			set idfile $config(driverIDPath)/new_id
 			if {![file exists $idfile]} {
-				Log "\nTrying to load driver \"$driverModule\""
+				Log "\nTrying to load driver \"$config(driverModule)\""
 				set loader /sbin/modprobe
 				Log " loader is: $loader"
 				if [file exists $loader] {
-					if [catch {set result [exec $loader -v $driverModule]} err] {
-						Log " Running \"$loader $driverModule\" gave an error:\n  $err"
+					if [catch {set result [exec $loader -v $config(driverModule) 2>/dev/null]} err] {
+						Log " Running \"$loader $config(driverModule)\" gave an error:\n  $err"
 					}
 				} else {
 					Log " /sbin/modprobe not found"
 				}
 			}
 			if [file exists $idfile] {
-				Log "Trying to add ID to driver \"$driverModule\""
-				catch {exec logger -p syslog.notice "usb_modeswitch: adding device ID $usb(idVendor):$usb(idProduct) to driver \"$driverModule\""}
-				catch {exec echo "$usb(idVendor) $usb(idProduct)" >$idfile}
+				Log "Trying to add ID to driver \"$config(driverModule)\""
+				catch {exec logger -p syslog.notice "usb_modeswitch: adding device ID $usb(idVendor):$usb(idProduct) to driver \"$config(driverModule)\"" 2>/dev/null}
+				catch {exec echo "$usb(idVendor) $usb(idProduct)" >$idfile 2>/dev/null}
 				after 600
 				set devList2 [glob -nocomplain /dev/ttyUSB* /dev/ttyACM* /dev/ttyHS*]
 				if {[llength $devList1] >= [llength $devList2]} {
 					Log " still no new serial devices found"
 				} else {
+					AddLastSeen "$usb(idVendor):$usb(idProduct)"
 					Log " driver successfully bound"
 				}
 			} else {
@@ -416,7 +413,7 @@ if [regexp {ok:$} $report] {
 if [regexp {ok:} $report] {
 	Log "Checking for AVOID_RESET_QUIRK attribute"
 	if [file exists $devdir/avoid_reset_quirk] {
-		if [catch {exec echo "1" >$devdir/avoid_reset_quirk} err] {
+		if [catch {exec echo "1" >$devdir/avoid_reset_quirk 2>/dev/null} err] {
 			Log " Error setting the attribute: $err"
 		} else {
 			Log " AVOID_RESET_QUIRK activated"
@@ -503,7 +500,7 @@ return 1
 # end of proc {MatchDevice}
 
 
-proc {ParseConfigFile} {} {
+proc {ParseGlobalConfig} {} {
 
 global logging noswitching
 
@@ -536,92 +533,89 @@ while {![eof $rc]} {
 Log "Using global config file: $configFile"
 
 }
-# end of proc {ParseConfigFile}
+# end of proc {ParseGlobalConfig}
 
 
+proc ParseDeviceConfig {configFile} {
+
+global config
+set config(driverModule) ""
+set config(driverIDPath) ""
+set config(waitBefore) ""
+set rc [open $configFile r]
+set lineList [split [read $rc] \n]
+close $rc
+foreach line $lineList {
+	regexp {DriverModule[[:blank:]]*=[[:blank:]]*"?(\w+)"?} $line d config(driverModule)
+	regexp {DriverIDPath[[:blank:]]*=[[:blank:]]*?"?([/\-\w]+)"?} $line d config(driverIDPath)
+	regexp {WaitBefore[[:blank:]]*=[[:blank:]]*?(\d+)} $line d config(waitBefore)
+}
+set config(waitBefore) [string trimleft $config(waitBefore) 0]
+
+}
+# end of proc {ParseDeviceConfig}
+
+
+proc {ConfigGet} {command config} {
+
+global settings
+
+switch $command {
+
+	list {
+		if [file exists $settings(dbdir)/configPack.tar.gz] {
+			Log "Found packed config collection $settings(dbdir)/configPack.tar.gz"
+			if [catch {set configList [exec tar -tzf $settings(dbdir)/configPack.tar.gz 2>/dev/null]} err] {
+				Log "Error: problem opening config package; tar returned\n $err"
+				return {}
+			}
+			set configList [split $configList \n]
+			set configList [lsearch -all -inline $configList $config*]
+		} else {
+			set configList [glob -nocomplain $settings(dbdir)/$config*]
+		}
+
+		return $configList
+	}
+	config {
+		if [file exists $settings(dbdir)/configPack.tar.gz] {
+			set settings(tmpConfig) /tmp/usb_modeswitch.current_cfg
+			Log "Extracting config $config from collection $settings(dbdir)/configPack.tar.gz"
+			set wc [open $settings(tmpConfig) w]
+			puts -nonewline $wc [exec tar -xzOf $settings(dbdir)/configPack.tar.gz $config 2>/dev/null]
+			close $wc
+		} else {
+			set settings(tmpConfig) $config
+		}
+		return $settings(tmpConfig)
+	}
+}
+
+}
+# end of proc {ConfigGet}
+
 proc {Log} {msg} {
 
 global wc logging device
 if {$logging == 0} {return}
 if {![info exists wc]} {
-	set wc [open /var/log/usb_modeswitch_$device a]
+	if [catch {set wc [open /var/log/usb_modeswitch_$device a]} err] {
+		set wc "error"
+		puts "Error: Can't write to log file, $err"
+		return
+	}
 	puts $wc "\n\nUSB_ModeSwitch log from [clock format [clock seconds]]\n"
 }
+if {$wc == "error"} {return}
 puts $wc $msg
 
 }
 # end of proc {Log}
 
 
-
-# Checking for interrupt endpoint in ttyUSB port; if found,
-# check for unused "gsmmodem[n]" name.
-# First link will be "gsmmodem", then "gsmmodem2" and up
-
-proc {SymLinkName} {path} {
-
-# HACK ... /tmp/gsmmodem_* was generated by a switching run before;
-# no way found to signal annother instance in the udev environment
-
-set idx -1
-set tmpNames [glob -nocomplain /tmp/gsmmodem_*]
-foreach tmpName $tmpNames {
-	set dev_top [lindex [split $tmpName _] 1]
-	set dirList [split $path /]
-	set idx [lsearch $dirList $dev_top]
-	if {$idx == -1} {
-		continue
-	} else {break}
-}
-if {$idx == -1} {return ""}
-
-regexp {ttyUSB\d+?} $path myPort
-
-#Log "symlink: dev_top is $dev_top \npath is $path \nport is $myPort"
-
-# Unfortunately, there are devices with more than one interrupt
-# port. We have to check all ports and assume that the lowest
-# number denotes the working port
-
-set devDir /sys[join [lrange $dirList 0 $idx] /]
-set portList {}
-foreach ifDir [glob -nocomplain $devDir/$dev_top:\[0-9\].\[0-9\]] {
-#	Log "ifDir is $ifDir"
-	if {![regexp {ttyUSB\d+?} [glob -nocomplain $ifDir/*] port]} {continue}
-	foreach epDir [glob -nocomplain $ifDir/ep_*] {
-		if [file exists $epDir/type] {
-			set rc [open $epDir/type r]
-			set type [read $rc]
-			close $rc
-			if [regexp {Interrupt} $type] {
-				lappend portList $port
-			}
-		}
-	}
-}
-set lowestPort [lindex [lsort -increasing $portList] 0]
-if {$lowestPort != $myPort} {
-	return ""
-}
-eval file delete dummy $tmpName
-cd /dev
-set idx 2
-set symlinkName "gsmmodem"
-while {$idx < 256} {
-	if {![file exists $symlinkName]} {
-		break
-	}
-	set symlinkName gsmmodem$idx
-	incr idx
-}
-
-return $symlinkName
-
-}
-# end of proc {SymLinkName}
-
-
+# Closing the log file if open and exit
 proc {SafeExit} {} {
+
 global wc
 if [info exists wc] {
 	catch {close $wc}
@@ -632,6 +626,178 @@ exit
 # end of proc {SafeExit}
 
 
+# Checking for interrupt endpoint in ttyUSB port (lowest if there is
+# more than one); if found, check for unused "gsmmodem[n]" name.
+# Link for first modem will be "gsmmodem", then "gsmmodem2" and up
+
+proc {SymLinkName} {path} {
+
+# Internal proc, used only here
+proc {hasInterrupt} {ifDir} {
+	if {[llength [glob -nocomplain $ifDir/ttyUSB*]] == 0} {return 0}
+	foreach epDir [glob -nocomplain $ifDir/ep_*] {
+		if [file exists $epDir/type] {
+			set rc [open $epDir/type r]
+			set type [read $rc]
+			close $rc
+			if [regexp {Interrupt} $type] {
+				return 1
+			}
+		}
+	}
+	return 0
+}
+
+# In case the device path is returned as /class/tty/ttyUSB,
+# we need to extract the USB device path from symlink "device"
+set linkpath /sys$path/device
+if [file exists $linkpath] {
+	if {[file type $linkpath] == "link"} {
+		set rawpath [file link $linkpath]
+		set trimpath [regsub -all {\.\./} $rawpath {}]
+		if [file isdirectory /sys/$trimpath] {
+			set path /$trimpath
+		}
+	}
+}
+
+if {![regexp {ttyUSB\d+?} $path myPort]} {
+	return ""
+}
+if {![regexp "\\d+\\.(\\d+)/$myPort" $path d myIf]} {
+	return ""
+}
+if {![regexp {usb\d*/(\d+-\d+)/} $path d dev_top]} {
+	return ""
+}
+
+set dirList [split $path /]
+set idx [lsearch $dirList $dev_top]
+set devDir /sys[join [lrange $dirList 0 $idx] /]
+
+if {![regexp "$devDir/$dev_top:\[0-9\]" /sys$path ifRoot]} {
+	return ""
+}
+set ifDir $ifRoot.$myIf
+
+set rightPort 0
+if [hasInterrupt $ifDir] {
+	set rightPort 1
+}
+
+# Unfortunately, there are devices with more than one interrupt
+# port. The assumption so far is that the lowest of these is
+# right. Check all lower interfaces for annother one (if interface)
+# is bigger than 0). If found, don't return any name.
+if { $rightPort && ($myIf > 0) } {
+	for {set i 0} {$i < $myIf} {incr i} {
+		set ifDir $ifRoot.$i
+		if [hasInterrupt $ifDir] {
+			set rightPort 0
+			break
+		}
+	}
+}
+if {$rightPort == 0} {
+	return ""
+}
+
+# Use first free "gsmmodem[n]" name
+cd /dev
+set idx 2
+set symlinkName "gsmmodem"
+while {$idx < 256} {
+	if {![file exists $symlinkName]} {
+		break
+	}
+	set symlinkName gsmmodem$idx
+	incr idx
+}
+return $symlinkName
+
+}
+# end of proc {SymLinkName}
+
+
+# Add serial driver after warm boot
+proc {CheckDriverBind} {path vid pid prod} {
+
+if {$vid == ""} {set vid $prod}
+if [regexp {/} $vid] {
+	set id_list [split $vid /]
+	set vid [format %04s [lindex $id_list 0]]
+	set pid [format %04s [lindex $id_list 1]]
+}
+
+set dirList [glob -nocomplain /sys$path/*]
+if [string match *ttyUSB* $dirList] {return}
+
+if {[WasLastSeen $vid:$pid] == 0} {return}
+
+set config(driverModule) "option"
+set config(driverIDPath) "/sys/bus/usb-serial/drivers/option1"
+set idfile $config(driverIDPath)/new_id
+if {![file exists $idfile]} {
+	set loader /sbin/modprobe
+	if {![file exists $loader]} {return}
+	if [catch {exec $loader $config(driverModule) 2>/dev/null}] {return}
+	set i 0
+	while {$i < 50} {
+		if [file exists $idfile] {
+			break
+		}
+		after 20
+		incr i
+	}
+	if {$i == 50} {return}
+}
+catch {exec echo "$vid $pid" >$idfile 2>/dev/null}
+
+}
+# end of proc {CheckDriverBind}
+
+
+# Add USB ID to list of devices needing driver binding
+proc {AddLastSeen} {id} {
+
+set lastseen /etc/usb_modeswitch.d/last_seen
+if [file exists $lastseen] {
+	set rc [open $lastseen r]
+	set buffer [read $rc]
+	close $rc
+	if [string match *$id* $buffer] {
+		return
+	}
+	set idList [split [string trim $buffer] \n]
+}
+lappend idList $id
+set buffer [join $idList "\n"]
+if [catch {set wc [open $lastseen w]}] {return}
+puts -nonewline $wc $buffer
+close $wc
+
+}
+# end of proc {AddLastSeen}
+
+
+# Check if USB ID is listed as needing driver binding
+proc {WasLastSeen} {id} {
+
+set lastseen /etc/usb_modeswitch.d/last_seen
+if {![file exists $lastseen]} {return 0}
+set rc [open $lastseen r]
+set buffer [read $rc]
+close $rc
+if [string match *$id* $buffer] {
+	return 1
+} else {
+	return 0
+}
+
+}
+# end of proc {WasLastSeen}
+
+
 # The actual entry point
 Main $argc $argv
 
